/************************************************************************
    MeOS - Orienteering Software
    Copyright (C) 2009-2010 Melin Software HB
    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Melin Software HB - software@melin.nu - www.melin.nu
    Stigbergsvgen 7, SE-75242 UPPSALA, Sweden
    
************************************************************************/

#include "stdafx.h"

#include "resource.h"

#include <commctrl.h>
#include <commdlg.h> 

#include "oEvent.h"
#include "xmlparser.h"
#include "gdioutput.h"
#include "csvparser.h"
#include "SportIdent.h"
#include <cassert>

#include "TabControl.h"

TabControl::TabControl(oEvent *poe):TabBase(poe)
{
  controlId = 0;
  assert(!FixedTabs::controlTab);
  FixedTabs::controlTab=this;
}

TabControl::~TabControl(void)
{ 
  FixedTabs::controlTab=0;
}

void TabControl::selectControl(gdioutput &gdi,  pControl pc)
{
	if (pc) {
		pc->synchronize();
		
    if (pc->getStatus() == oControl::StatusStart ||
          pc->getStatus() == oControl::StatusFinish) {
      gdi.selectItemByData("Controls", pc->getId());
      gdi.selectItemByData("Status", oControl::StatusOK);
		  gdi.setText("ControlID", "-", true);

		  gdi.setText("Code", "");
		  gdi.setText("Name", pc->getName());
      gdi.setText("TimeAdjust", "00:00");
      gdi.setText("MinTime", "-");
      controlId = pc->getId();
  	
		  gdi.enableInput("Remove");
		  gdi.enableInput("Save");
      gdi.enableEditControls(false);
      gdi.enableInput("Name");
    }
    else {
      gdi.selectItemByData("Controls", pc->getId());
		  gdi.selectItemByData("Status", pc->getStatus());

		  gdi.setText("ControlID", pc->getId(), true);

		  gdi.setText("Code", pc->codeNumbers());
		  gdi.setText("Name", pc->getName());
      gdi.setText("TimeAdjust", pc->getTimeAdjustS());
      gdi.setText("MinTime", pc->getMinTimeS());

      controlId = pc->getId();
  	
		  gdi.enableInput("Remove");
		  gdi.enableInput("Save");
      gdi.enableEditControls(true);
    }
	}
	else{
    gdi.selectItemByData("Controls", -1);
    gdi.selectItemByData("Status", oControl::StatusOK);
		gdi.setText("Code", "");
		gdi.setText("Name", "");
    controlId = 0;
	
		gdi.setText("ControlID", "-", true);
    gdi.setText("TimeAdjust", "00:00");

		gdi.disableInput("Remove");
		gdi.disableInput("Save");
    gdi.enableEditControls(false);
	}
}

int ControlsCB(gdioutput *gdi, int type, void *data)
{
  if (!FixedTabs::controlTab) 
		throw std::exception("tabControl undefined.");

  return FixedTabs::controlTab->controlCB(*gdi, type, data);
}

void TabControl::save(gdioutput &gdi)
{
	DWORD pcid = controlId;
	bool create = controlId==0;

	pControl pc;
	if( pcid==0 )
		pc = oe->addControl(0, 0, "");
	else 
    pc = oe->getControl(pcid, false);

  if(!pc) 
    throw std::exception("Internal error");
  if (pc->getStatus() != oControl::StatusFinish && pc->getStatus() != oControl::StatusStart) {
	  if(!pc->setNumbers(gdi.getText("Code")))
		  gdi.alert("Kodsiffran mste vara ett heltal. Flera kodsiffror mste separeras med komma.");
  					
	  ListBoxInfo lbi;
	  gdi.getSelectedItem("Status", &lbi);
    pc->setStatus(oControl::ControlStatus(lbi.data));
    pc->setTimeAdjust(gdi.getText("TimeAdjust"));
    pc->setMinTime(gdi.getText("MinTime"));
  }

	pc->setName(gdi.getText("Name"));
  
	pc->synchronize();
	oe->fillControls(gdi, "Controls");

  oe->reEvaluateAll(true);

	if(create)	selectControl(gdi, 0);
	else		selectControl(gdi, pc);
}

int TabControl::controlCB(gdioutput &gdi, int type, void *data)
{
	if (type==GUI_BUTTON) {
		ButtonInfo bi=*(ButtonInfo *)data;

		if (bi.id=="Save")
      save(gdi);
		else if (bi.id=="Add") {
      if (controlId>0)
        save(gdi);
      pControl pc = oe->addControl(0,oe->getNextControlNumber(), ""); 
			oe->fillControls(gdi, "Controls");
			selectControl(gdi, pc);
		}
		else if (bi.id=="Remove") {

			DWORD cid = controlId;
			if (cid==0) {
				gdi.alert("Ingen kontroll vald vald.");
				return 0;
			}

			if(oe->isControlUsed(cid))
				gdi.alert("Kontrollen anvnds och kan inte tas bort.");
			else
				oe->removeControl(cid);

			oe->fillControls(gdi, "Controls");
			selectControl(gdi, 0);
		}
	}
	else if(type==GUI_LISTBOX){
		ListBoxInfo bi=*(ListBoxInfo *)data;

		if (bi.id=="Controls") {
    
      if (gdi.isInputChanged(""))
        save(gdi);

			pControl pc=oe->getControl(bi.data, false);
      if(!pc)
        throw std::exception("Internal error");

			selectControl(gdi, pc);
		}
	}
  else if(type==GUI_CLEAR) {
    if (controlId>0)
      save(gdi);

    return true;
  }
	return 0;
}


bool TabControl::loadPage(gdioutput &gdi)
{
  oe->checkDB();
	gdi.selectTab(tabId);

	gdi.clearPage(false);
	gdi.fillDown();
	gdi.addString("", boldLarge, "Kontroller");

	gdi.pushY();
  gdi.addListBox("Controls", 220, 400, ControlsCB).isEdit(false).ignore(true);
	gdi.setTabStops("Controls", 150);
	oe->fillControls(gdi, "Controls");

	gdi.newColumn();
	gdi.fillDown();
	gdi.popY();

  gdi.pushX();
	gdi.fillRight();
  gdi.addString("", 1, "Kontrollens ID-nummer:");
	gdi.fillDown();
  gdi.addString("ControlID", 1, "#-").setColor(200,50,50);
  gdi.popX();

  gdi.fillRight();
	gdi.addInput("Name", "", 16, 0, "Kontrollnamn:");

  gdi.addSelection("Status", 100, 100, ControlsCB, "Status:", "Ange om kontrollen fungerar och hur den ska anvndas.");
  gdi.addItem("Status", lang.tl("OK"), oControl::StatusOK);
	gdi.addItem("Status", lang.tl("Multipel"), oControl::StatusMultiple);
  gdi.addItem("Status", lang.tl("Trasig"), oControl::StatusBad);
	
	gdi.addInput("TimeAdjust", "", 6, 0, "Tidsjustering:");
	gdi.fillDown();
  gdi.addInput("MinTime", "", 6, 0, "Minsta strcktid:");
	
  gdi.popX();
  gdi.dropLine(0.5);
	gdi.addString("", 10, "help:9373");
  gdi.dropLine(0.5);
	gdi.addInput("Code", "", 16);

  gdi.popX();
	gdi.dropLine(2);

  gdi.fillRight();	
	gdi.addButton("Save", "Spara", ControlsCB, "help:save");	
  gdi.disableInput("Save");
	gdi.addButton("Remove", "Radera", ControlsCB);
	gdi.disableInput("Remove");
	gdi.addButton("Add", "Ny kontroll", ControlsCB);

  gdi.dropLine(2);
	gdi.popX();
  gdi.fillDown();
  gdi.dropLine();
	gdi.addString("", 10, "help:89064");
  
  selectControl(gdi, oe->getControl(controlId, false));

  gdi.setOnClearCb(ControlsCB);

  gdi.refresh();
  return true;	
}